@@ -11,8 +11,8 @@ class JPEGFormat : public ImageFormat {
         bool can_read() const { return true; }
         bool can_write() const { return true; }
 
-        std::auto_ptr<Image> read(byte_source* src, ImageFactory* factory);
-        void write(Image* input, byte_sink* output);
+        std::auto_ptr<Image> read(byte_source* src, ImageFactory* factory, const options_map& opts);
+        void write(Image* input, byte_sink* output, const options_map& opts);
 };
 
 
