@@ -8,12 +8,21 @@
 
 #include <sstream>
 #include <iostream>
+#include <cstdio>
+#include <cstring>
 
 extern "C" {
    #include <tiffio.h>
 }
 
 namespace {
+
+void show_tiff_warning(const char* module, const char* fmt, va_list ap) {
+    std::fprintf(stderr, "%s: ", module);
+    std::vfprintf(stderr, fmt, ap);
+    std::fprintf(stderr, "\n");
+}
+
 tsize_t tiff_read(thandle_t handle, void* data, tsize_t n) {
     byte_source* s = static_cast<byte_source*>(handle);
     return s->read(static_cast<byte*>(data), n);
@@ -58,14 +67,35 @@ void tiff_error(const char* module, const char* fmt, va_list ap) {
     throw CannotReadError(std::string("imread._tiff: libtiff error: `") + buffer + std::string("`"));
 }
 
+
 struct tif_holder {
     tif_holder(TIFF* tif)
         :tif(tif)
         { }
-    ~tif_holder() { TIFFClose(tif); }
+
+    ~tif_holder() {
+        TIFFClose(tif);
+    }
+
     TIFF* tif;
 };
 
+struct tiff_warn_error {
+    tiff_warn_error()
+        :warning_handler_(TIFFSetWarningHandler(show_tiff_warning))
+        ,error_handler_(TIFFSetErrorHandler(tiff_error))
+    { }
+    ~tiff_warn_error() {
+        TIFFSetWarningHandler(warning_handler_);
+        TIFFSetErrorHandler(error_handler_);
+    }
+
+    // Newer versions of TIFF seem to call this TIFFWarningHandler, but older versions do not have this type
+    typedef void (*tiff_handler_type)(const char* module, const char* fmt, va_list ap);
+    tiff_handler_type warning_handler_;
+    tiff_handler_type error_handler_;
+};
+
 template <typename T>
 inline
 T tiff_get(const tif_holder& t, const int tag) {
@@ -108,7 +138,6 @@ std::string tiff_get<std::string>(const tif_holder& t, const int tag, const std:
 }
 
 TIFF* read_client(byte_source* src) {
-    TIFFSetErrorHandler(tiff_error);
     return TIFFClientOpen(
                     "internal",
                     "r",
@@ -179,6 +208,8 @@ struct stk_extend {
 std::auto_ptr<image_list> STKFormat::read_multi(byte_source* src, ImageFactory* factory) {
     shift_source moved(src);
     stk_extend ext;
+    tiff_warn_error twe;
+
     tif_holder t = read_client(&moved);
     std::auto_ptr<image_list> images(new image_list);
     const uint32 h = tiff_get<uint32>(t, TIFFTAG_IMAGELENGTH);
@@ -216,6 +247,7 @@ std::auto_ptr<image_list> STKFormat::read_multi(byte_source* src, ImageFactory*
 }
 
 std::auto_ptr<image_list> TIFFFormat::do_read(byte_source* src, ImageFactory* factory, bool is_multi) {
+    tiff_warn_error twe;
     tif_holder t = read_client(src);
     std::auto_ptr<image_list> images(new image_list);
     do {
@@ -241,7 +273,7 @@ std::auto_ptr<image_list> TIFFFormat::do_read(byte_source* src, ImageFactory* fa
 }
 
 void TIFFFormat::write(Image* input, byte_sink* output, const options_map& opts) {
-    TIFFSetErrorHandler(tiff_error);
+    tiff_warn_error twe;
     tif_holder t = TIFFClientOpen(
                     "internal",
                     "w",
@@ -253,7 +285,9 @@ void TIFFFormat::write(Image* input, byte_sink* output, const options_map& opts)
                     tiff_size<byte_sink>,
                     NULL,
                     NULL);
-
+    std::vector<uint8_t> bufdata;
+    void* bufp = 0;
+    bool copy_data = false;
     const uint32 h = input->dim(0);
     const uint16 photometric = ((input->ndims() == 3 && input->dim(2)) ?
                                                     PHOTOMETRIC_RGB :
@@ -268,7 +302,23 @@ void TIFFFormat::write(Image* input, byte_sink* output, const options_map& opts)
     TIFFSetField(t.tif, TIFFTAG_PHOTOMETRIC, uint16(photometric));
     TIFFSetField(t.tif, TIFFTAG_PLANARCONFIG, PLANARCONFIG_CONTIG);
 
-    TIFFSetField(t.tif, TIFFTAG_COMPRESSION, COMPRESSION_LZW);
+    if (get_optional_bool(opts, "tiff:compress", true)) {
+        TIFFSetField(t.tif, TIFFTAG_COMPRESSION, COMPRESSION_LZW);
+        // For 8 bit images, prediction defaults to false; for 16 bit images,
+        // it defaults to true. This is because compression of raw 16 bit
+        // images is often counter-productive without this flag. See the
+        // discusssion at http://www.asmail.be/msg0055176395.html
+        const bool prediction_default = input->nbits() != 8;
+        if (get_optional_bool(opts, "tiff:horizontal-predictor", prediction_default)) {
+            TIFFSetField(t.tif, TIFFTAG_PREDICTOR, PREDICTOR_HORIZONTAL);
+            if (!copy_data) {
+                bufdata.resize(input->dim(1) * input->nbytes());
+                bufp = &bufdata[0];
+                copy_data = true;
+            }
+        }
+    }
+
     TIFFSetField(t.tif, TIFFTAG_ORIENTATION, ORIENTATION_TOPLEFT);
     const char* meta = get_optional_cstring(opts, "metadata");
     if (meta) {
@@ -304,9 +354,15 @@ void TIFFFormat::write(Image* input, byte_sink* output, const options_map& opts)
     }
 
     for (uint32 r = 0; r != h; ++r) {
-        if (TIFFWriteScanline(t.tif, input->rowp(r), r) == -1) {
+        void* rowp = input->rowp(r);
+        if (copy_data) {
+            std::memcpy(bufp, rowp, input->dim(1) * input->nbytes());
+            rowp = bufp;
+        }
+        if (TIFFWriteScanline(t.tif, rowp, r) == -1) {
             throw CannotWriteError("imread.imsave._tiff: Error writing TIFF file");
         }
     }
     TIFFFlush(t.tif);
 }
+
